<?php
/*
Plugin Name: Author Lists
plugin URI: https://99robots.com/products/list-authors/
Description: List Authors from your blog in a widget, shortcode, or php template tags.
version: 1.0.6
Author: 99 Robots
Author URI: https://99robots.com
License: GPL2
*/

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

/* Item Name */

if (!defined('NNROBOTS_LIST_AUTHORS_ITEM_NAME'))
    define('NNROBOTS_LIST_AUTHORS_ITEM_NAME', 'Author Lists');

/* Store URL */

if (!defined('NNROBOTS_LIST_AUTHORS_STORE_URL'))
    define('NNROBOTS_LIST_AUTHORS_STORE_URL', 'https://99robots.com');

/* Minimum PHP version */

if (!defined('NNROBOTS_LIST_AUTHORS_MIN_PHP_VERSION'))
    define('NNROBOTS_LIST_AUTHORS_MIN_PHP_VERSION', '5.4.0');

/* Plugin Name */

if (!defined('NNROBOTS_LIST_AUTHORS_PLUGIN_NAME'))
    define('NNROBOTS_LIST_AUTHORS_PLUGIN_NAME', trim(dirname(plugin_basename(__FILE__)), '/'));

/* Plugin directory */

if (!defined('NNROBOTS_LIST_AUTHORS_PLUGIN_DIR'))
    define('NNROBOTS_LIST_AUTHORS_PLUGIN_DIR', plugin_dir_path(__FILE__) );

/* Plugin url */

if (!defined('NNROBOTS_LIST_AUTHORS_PLUGIN_URL'))
    define('NNROBOTS_LIST_AUTHORS_PLUGIN_URL', plugins_url() . '/' . NNROBOTS_LIST_AUTHORS_PLUGIN_NAME);

/* Plugin verison */

if (!defined('NNROBOTS_LIST_AUTHORS_VERSION_NUM'))
    define('NNROBOTS_LIST_AUTHORS_VERSION_NUM', '1.0.6');

// Activatation / Deactivation

register_activation_hook( __FILE__, 			array('NNRobots_List_Author', 'register_activation'));

// Shortcode

add_action('media_buttons', 					array('NNRobots_List_Author', 'add_media_buttons'), 20);
add_action('wp_enqueue_media', 					array('NNRobots_List_Author', 'include_shortcode_button_scripts'));
add_shortcode('nnr_author_list',				'nnrobots_author_list_shortcode');

// Hooks / Filter

add_action('admin_menu', 						array('NNRobots_List_Author', 'menu'));
add_action('init', 								array('NNRobots_List_Author', 'load_textdoamin'));

$plugin = plugin_basename(__FILE__);
add_filter("plugin_action_links_$plugin", 		array('NNRobots_List_Author', 'settings_link'));

// Include widget

require_once('list-author-widget.php');

// Software Licensing Updates

add_action('wp_ajax_nnr_la_license_action', 	array('NNRobots_List_Author', 'license_action'));

if ( !class_exists('EDD_SL_Plugin_Updater') ) {
	include_once( dirname( __FILE__ ) . '/admin/include/edd-software-licensing-updates/EDD_SL_Plugin_Updater.php' );
}

$edd_updater = new EDD_SL_Plugin_Updater( NNROBOTS_LIST_AUTHORS_STORE_URL , __FILE__, array(
		'version' 	=> NNROBOTS_LIST_AUTHORS_VERSION_NUM,
		'license' 	=> NNRobots_List_Author::get_license_key(),
		'item_name' => NNROBOTS_LIST_AUTHORS_ITEM_NAME,
		'author' 	=> '99 Robots',
	)
);

/**
 * NNRobots_List_Author class.
 */
class NNRobots_List_Author {

	/**
	 * prefix
	 *
	 * (default value: 'nnrobots_author_lists_')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $prefix = 'nnrobots_author_lists_';

	/**
	 * prefix_dash
	 *
	 * (default value: 'nnr-al-')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $prefix_dash = 'nnr-al-';

	/**
	 * settings_page
	 *
	 * (default value: 'nnrobots-list-authors-settings')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $settings_page = 'nnrobots-list-authors-settings';

	/**
	 * text_domain
	 *
	 * (default value: '99robotos-list-author')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	static $text_domain = '99robotos-author-lists';

	/**
	 * Performs tasks needed upon activation
	 *
	 * @since 1.0.0
	 */
	static function register_activation() {

		/* Check if multisite, if so then save as site option */

		if (is_multisite()) {
			add_site_option(self::$prefix . 'version', NNROBOTS_LIST_AUTHORS_VERSION_NUM);
		} else {
			add_option(self::$prefix . 'version', NNROBOTS_LIST_AUTHORS_VERSION_NUM);
		}
	}

	/**
	 * Load the text domain
	 *
	 * @since 1.0.0
	 */
	static function load_textdoamin() {
		load_plugin_textdomain(self::$text_domain, false, basename( dirname( __FILE__ ) ) . '/languages' );
	}

	/**
	 * Hooks to 'plugin_action_links_' filter
	 *
	 * @since 1.0.0
	 */
	static function settings_link($links) {
		$settings_link = '<a href="admin.php?page=' . self::$settings_page . '">Settings</a>';
		array_unshift($links, $settings_link);

		$widgets_link = '<a href="widgets.php">Widget</a>';
		array_unshift($links, $widgets_link);

		return $links;
	}

	/**
	 * Perform all the actions with a license
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function license_action() {

		if ( !isset($_POST['license_key']) ) {
			echo '';
			die();
		}

		// data to send in our API request

		$api_params = array(
			'edd_action'	=> $_POST['license_action'],
			'license' 		=> $_POST['license_key'],
			'item_name' 	=> urlencode( NNROBOTS_LIST_AUTHORS_ITEM_NAME ), // the name of our product in EDD
			'url'       	=> is_multisite() ? network_home_url() : home_url()
		);

		// Call the custom API.

		$response = wp_remote_post( NNROBOTS_LIST_AUTHORS_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay

		if ( is_wp_error( $response ) ) {
			echo $response;
			die();
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		if ( $_POST['license_action'] == 'activate_license' ) {
			self::update_license_key($_POST['license_key']);
		} else if ( $_POST['license_action'] == 'deactivate_license' && $license_data->license == 'deactivated') {
			self::update_license_key('');
		}

		// decode the license data

		echo $license_data->license;

		die();

	}

	/**
	 * Check if license key is active
	 *
	 * @access public
	 * @static
	 * @param mixed $license_key
	 * @return void
	 */
	static function is_license_active( $license_key ) {

		if ( !isset( $license_key ) ) {
			return false;
		}

		// data to send in our API request

		$api_params = array(
			'edd_action'	=> 'check_license',
			'license' 		=> $license_key,
			'item_name' 	=> urlencode( NNROBOTS_LIST_AUTHORS_ITEM_NAME ), // the name of our product in EDD
			'url'       	=> is_multisite() ? network_home_url() : home_url()
		);

		// Call the custom API.

		$response = wp_remote_post( NNROBOTS_LIST_AUTHORS_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay

		if ( is_wp_error( $response ) ) {
			return false;
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// decode the license data

		if ( isset($license_data->license) && $license_data->license == 'valid' ) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Get the license key
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function get_license_key() {

		// Get the license key from based on WordPress install

		if ( function_exists('is_multisite') && is_multisite() ) {
			$license_key = get_site_option(self::$prefix . 'license_key');
		} else {
			$license_key = get_option(self::$prefix . 'license_key');
		}

		// Check if license key is set

		if ( $license_key === false ) {
			$license_key = '';
		}

		return $license_key;
	}

	/**
	 * Update the license key
	 *
	 * @access public
	 * @static
	 * @param mixed $license_key
	 * @return void
	 */
	static function update_license_key($license_key) {

		if ( isset( $license_key ) ) {

			// Get the license key from based on WordPress install

			if ( function_exists('is_multisite') && is_multisite() ) {

				update_site_option(self::$prefix . 'license_key', $license_key);

			} else {

				update_option(self::$prefix . 'license_key', $license_key);

			}
		}
	}

	/**
	 * Add a media button for adding Optin Fire shortcode
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function add_media_buttons() {

		echo '<button class="button ' . self::$prefix_dash . 'shortcode-builder"><span></span>' . __('Author List', self::$text_domain) . '</button>
		<div class="' . self::$prefix_dash . 'shortcode-builder-container">
		<div class="postbox ' . self::$prefix_dash . 'shortcode-builder-box" style="display:none;">
			<h3>
				<span>' . __('Author List Shortcode Builder', self::$text_domain) . '</span>

				<button style="float: right;margin-right: 0px;" class="button ' . self::$prefix_dash . 'shortcode-builder-close">' . __('Close', self::$text_domain) . '</button>
				<button style="float: right;margin-right: 12px;" class="button button-primary ' . self::$prefix_dash . 'shortcode-builder-add">' . __('Insert Shortcode', self::$text_domain) . '</button>
			</h3>
			<div class="inside">
				<p style="display:none;">
					<strong><label for="' . self::$prefix_dash . 'shortcode">' . __( 'ShortCode:', self::$text_domain). '</label></strong>
					<code class="' . self::$prefix_dash . 'shortcode">[nnr_author_list]</code>
				</p>

				<!-- Title -->

				<p>
					<label for="' . self::$prefix_dash . 'title">' . __('Title:', NNRobots_List_Author::$text_domain ) . '</label>
					<input class="widefat ' . self::$prefix_dash . 'shortcode-input" data-shortcode="title" id="' . self::$prefix_dash . 'title" name="' . self::$prefix_dash . 'title" type="text" value="' . __('Author List', self::$text_domain) . '" />
				</p>

				<!-- Layout -->

				<p>
					<label>' . __('Layout:' ,NNRobots_List_Author::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'layout" name="' . self::$prefix_dash . 'layout" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="layout">
						<option value="list" selected="selected">' . __('List', NNRobots_List_Author::$text_domain) . '</option>
						<option value="list-bio">' . __('List with Bio', NNRobots_List_Author::$text_domain) . '</option>
						<option value="grid">' . __('Grid', NNRobots_List_Author::$text_domain) . '</option>
						<option value="carousel">' . __('Carousel', NNRobots_List_Author::$text_domain) . '</option>
					</select>
				</p>

				<!-- Number of Authors to display -->

				<p>
					<label>' . __('Number of Authors to display:' ,NNRobots_List_Author::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'num-authors" name="' . self::$prefix_dash . 'num-authors" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="num_authors">
						<option value="all" selected="selected">' . __('All', NNRobots_List_Author::$text_domain) . '</option>';

						for ($i = 1; $i < 21; $i++) {
							echo '<option value="' . $i . '">' . __($i, NNRobots_List_Author::$text_domain) . '</option>';
						}

					echo '</select>
				</p>

				<!-- User Roles -->

				<p>
					<label>' . __('User Roles' ,NNRobots_List_Author::$text_domain) . '</label>
					<input id="' . self::$prefix_dash . 'user-roles" name="' . self::$prefix_dash . 'user-roles" type="text" class="widefat ' . self::$prefix_dash . 'user-roles ' . self::$prefix_dash . 'shortcode-input" data-shortcode="user_roles" value="Administrator,Editor,Author" />
					<em>' . __('Which users should be shown in the Author List?', NNRobots_List_Author::$text_domain) . '</em>
				</p>

				<!-- Exclude Users -->

				<p>
					<label>' . __('Exclude Users', NNRobots_List_Author::$text_domain) . '</label>
					<input id="' . self::$prefix_dash . 'exclude-users" name="' . self::$prefix_dash . 'exclude-users" type="text" class="widefat ' . self::$prefix_dash . 'exclude-users ' . self::$prefix_dash . 'shortcode-input" data-shortcode="exclude_users" value="" />
					<em>' . __('Do not show this users in the list.', NNRobots_List_Author::$text_domain) . '</em>
				</p>

				<!-- Sort by -->

				<p>
					<label>' . __('Sort by:' ,NNRobots_List_Author::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'orderby" name="' . self::$prefix_dash . 'orderby" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="orderby">
						<option value="ID" selected="selected">' . __('ID' ,NNRobots_List_Author::$text_domain) . '</option>
						<option value="display_name">' . __('Alphabetically' ,NNRobots_List_Author::$text_domain) . '</option>
						<option value="post_count">' . __('Post Count' ,NNRobots_List_Author::$text_domain) . '</option>
					</select>
				</p>

				<!-- Order by -->

				<p>
					<label>' . __('Order by:' ,NNRobots_List_Author::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'order" name="' . self::$prefix_dash . 'order" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="order"">
						<option value="ASC" selected="selected">' . __('ASC' ,NNRobots_List_Author::$text_domain) . '</option>
						<option value="DESC">' . __('DESC' ,NNRobots_List_Author::$text_domain) . '</option>
					</select>
				</p>

				<!-- Avatar Size -->

				<p>
					<label>' . __('Avatar Size:' ,NNRobots_List_Author::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'avatar-size" name="' . self::$prefix_dash . 'avatar-size" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="avatar_size"">
						<option value="small">' . __('Small', NNRobots_List_Author::$text_domain) . '</option>
						<option value="medium" selected="selected">' . __('Medium', NNRobots_List_Author::$text_domain) . '</option>
						<option value="large">' . __('Large', NNRobots_List_Author::$text_domain) . '</option>
						<option value="full">' . __('Full', NNRobots_List_Author::$text_domain) . '</option>
					</select>
				</p>

				<!-- Avatar Shape -->

				<p>
					<label>' . __('Avatar Shape:' ,NNRobots_List_Author::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'avatar-shape" name="' . self::$prefix_dash . 'avatar-shape" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="avatar_shape">
						<option value="rounded-square" selected="selected">' . __('Rounded Square', NNRobots_List_Author::$text_domain) . '</option>
						<option value="square">' . __('Square', NNRobots_List_Author::$text_domain) . '</option>
						<option value="circle">' . __('Circle', NNRobots_List_Author::$text_domain) . '</option>
					</select>
				</p>

			</div>
		</div>
		</div>';

	}

	/**
	 * Include the shortcode builder stylesheet and script
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function include_shortcode_button_scripts() {

		if ( isset($_GET['post']) ) {

			// Styles

			wp_enqueue_style(self::$prefix . 'shortcode_builder_css', 		NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/admin/css/shortcode-builder.css');
			wp_enqueue_style('wp-color-picker');

			// Scripts

			global $wp_roles;
			$roles = array();

			foreach ( $wp_roles->roles as $role ) {

				if ( isset($role['capabilities']['edit_posts']) && $role['capabilities']['edit_posts'] ) {
					$roles[] = $role['name'];
				}

			}

			wp_enqueue_script('wp-color-picker');
			wp_enqueue_style(self::$prefix . 'selectize_css', 				NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/admin/css/selectize.css');
			wp_enqueue_script(self::$prefix . 'selectize_js', 				NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/admin/js/selectize.min.js');
			wp_enqueue_script(self::$prefix . 'shortcode_builder_js', 		NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/admin/js/shortcode-builder.js', array('jquery'));
			wp_localize_script(self::$prefix . 'shortcode_builder_js', 		'nnr_al_shortcode', array(
				'prefix' 				=> self::$prefix_dash,
				'post_types'			=> get_post_types(array('public' => true), 'names'),
				'user_roles'			=> $roles,
				'users'					=> get_users(array('who'=>'authors', 'number' => 500)),
			));
		}

	}

	/**
	 * Hook for the admin menu
	 *
	 * @since 1.0.0
	 *
	 * @param 	N/A
	 * @return 	N/A
	 */
	static function menu() {

		// Settings page

		$settings_page_load = add_submenu_page(
	 		'options-general.php',
	 		__("Author Lists", self::$text_domain),
	 		__("Author Lists", self::$text_domain),
	 		'manage_options',
	 		self::$settings_page,
	 		array('NNRobots_List_Author','settings'));

	 	add_action("admin_print_scripts-$settings_page_load", array('NNRobots_List_Author', 'admin_include_scripts'));
	}

	/**
	 * include_scripts function.
	 *
	 * Include css on admin
	 * @access public
	 * @static
	 * @return void
	 */
	static function admin_include_scripts() {

		// Styles

		wp_enqueue_style(self::$prefix . 'settings_css', 	NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/admin/css/settings.css');
		wp_enqueue_style(self::$prefix . 'bootstrap_css', 	NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/admin/css/nnr-bootstrap.min.css');
		wp_enqueue_style(self::$prefix . 'fontawesome_css',	NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/admin/include/fontawesome/css/font-awesome.min.css');

		// Scripts

		wp_enqueue_script(self::$prefix . 'bootstrap_js', 	NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/admin/js/bootstrap.min.js', array('jquery'));
		wp_enqueue_script(self::$prefix . 'settings_js', 	NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/admin/js/settings.js', array('jquery'));
		wp_localize_script(self::$prefix . 'settings_js', 	'nnr_la_data', array(
			'prefix_dash'			=> self::$prefix_dash,
			'activate_text'			=> __('Activate', self::$text_domain),
			'deactivate_text'		=> __('Deactivate', self::$text_domain),
			'no_license_key'		=> __('Please add or activate your License Key to get automatic updates.  Without a valid license key you will not receive regular updates. You can find your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>',
			'expired'				=> __('EXPIRED: This license key has expired.  Please renew your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>',
			'activation_error'		=> __('INVALID LICENSE KEY: The license key is not valid please try again. You can find your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>.',
			'activation_valid'		=> __('SUCCESS: Your license key is valid.', self::$text_domain),
			'deactivation_valid'	=> __('SUCCESS: This site has been deactivated.', self::$text_domain),
			'deactivation_error'	=> __('DEACTIVATION FAILED: This site could not be deactivated.', self::$text_domain),
		));
	}

	/**
	 * Displays on the dashboard
	 *
	 * @since 1.0.0
	 *
	 * @param 	N/A
	 * @return 	N/A
	 */
	static function settings() {

		global $wp_version;

		$license_key = self::get_license_key();

		require('admin/views/settings.php');
	}
}

/**
 * Create shortcode [nnr_author_list ...]
 *
 * @since 1.0.0
 *
 * @param 	array of parameters
 * @return 	html code
 */
function nnrobots_author_list_shortcode($atts) {

	$args = shortcode_atts( array(
		'title' 		=> __('Author List', NNRobots_List_Author::$text_domain),
		'layout'		=> 'list',
		'num_authors'	=> '5',
		'user_roles'	=> 'Administrator,Editor,Author',
		'exclude_users'	=> '',
		'orderby' 		=> 'display_name',
		'order'			=> 'ASC',
		'avatar_size'	=> 'medium',
		'avatar_shape'	=> 'rounded-square',
	), $atts );

	return NNRobots_List_Author_Widget::display_author_list(null, $args, true, NNRobots_List_Author::$prefix_dash . 'shortcode');
}
