<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

// Activate Widget

add_action('widgets_init', 	array('NNRobots_List_Author_Widget', 'register_widget'));

/**
 * NNRobots_List_Author_Widget class.
 *
 * @extends WP_Widget
 */
class NNRobots_List_Author_Widget extends WP_Widget {

	/**
	 * Register widget
	 *
	 * @since 1.0.0
	 *
	 * @param 	N/A
	 * @return 	N/A
	 */
	static function register_widget() {
		register_widget( 'NNRobots_List_Author_Widget' );
	}

	/**
	 * Register widget with WordPress.
	 */
	function __construct() {
		parent::__construct(
			NNRobots_List_Author::$prefix, 																				// Base ID
			__('Author Lists', NNRobots_List_Author::$text_domain), 													// Name
			array( 'description' => __( 'Display of list of all site authors.' , NNRobots_List_Author::$text_domain)) 	// Args
		);
	}

	/**
	 * Front-end display of widget.
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args, $instance ) {
		echo self::display_author_list($args, $instance, false, $this->id);
	}

	/**
	 * Display the author list
	 *
	 * @access public
	 * @static
	 * @param mixed $instance
	 * @param bool $shortcode (default: false)
	 * @return void
	 */
	static function display_author_list($args, $instance, $shortcode = false, $this_id = '') {

		$title            = isset($instance['title']) ? $instance['title'] : '';
		$layout           = isset($instance['layout']) ? $instance['layout'] :'';
		$user_roles       = isset($instance['user_roles']) ? $instance['user_roles'] : '';
		$exclude_users    = isset($instance['exclude_users']) ? $instance['exclude_users'] : '';
		$orderby          = isset($instance['orderby']) ? $instance['orderby'] : '';
		$order            = isset($instance['order']) ? $instance['order'] : '';
		$avatar_size      = isset($instance['avatar_size']) ? $instance['avatar_size'] : '';
		$avatar_shape     = isset($instance['avatar_shape']) ? $instance['avatar_shape'] : '';
		$num_authors      = isset($instance['num_authors']) ? $instance['num_authors'] : '';

		$authors = get_users(array(
			'blog_id'      => $GLOBALS['blog_id'],
			'orderby'      => $orderby,
			'order'        => $order,
			'count_total'  => false,
			'fields'       => 'all',
			'who' 		   => 'contributors',
			'exclude'	   => explode(',', $exclude_users),
		));

		$valid_authors = array();

		foreach ( $authors as $author ) {
			foreach ( explode(',', $user_roles) as $role) {
				if ( in_array( strtolower($role) , $author->roles) ) {
					$valid_authors[] = $author;
					break;
				}
			}
		}

		if ( $num_authors == 'all' ) {
			$num_authors = 50;
		}

		$authors = array();
		$i = 1;
		foreach ( $valid_authors as $author ) {
			if ( $i > $num_authors ) {
				break;
			} else {
				$authors[] = $author;
			}
			$i++;
		}

		$code = '';

		if ( !$shortcode ) {
			$code .= $args['before_widget'];

			if ( ! empty( $title ) ) {
				$code .= $args['before_title'] . $title . $args['after_title'];
			}

		} else {
			if ( ! empty( $title ) ) {
				$code .= '<h3>' . $title . '</h3>';
			}
		}

		$code .= '<!-- Author Lists Wrap -->

			<div ' . ($shortcode ? 'id="' . NNRobots_List_Author::$prefix_dash . 'shortcode"' : '') . ' class="' . NNRobots_List_Author::$prefix_dash . 'wrap" data-source="' . ($shortcode ? 'shortcode' : 'widget') . '" data-layout="' . $layout . '" data-avatar-shape="' . $avatar_shape . '">';

			if ( $layout == 'carousel' ) {
				$code .= '<div class="' . NNRobots_List_Author::$prefix_dash . 'slider flexslider"><ul class="slides">';
			}

			foreach ($authors as $author) {

				$posts = get_posts(array(
					'posts_per_page'   => 1,
					'offset'           => 0,
					'orderby'          => 'post_date',
					'order'            => 'DESC',
					'post_type'        => 'post',
					'post_status'      => 'publish',
					'suppress_filters' => true,
					'author'		   => $author->ID
				));

				switch ( $layout ) {

					case 'list-bio':

						$code .= '<div class="' . NNRobots_List_Author::$prefix_dash . 'author-list-item">

							<div class="' . NNRobots_List_Author::$prefix_dash . 'author-info-container" data-avatar-size="' . $avatar_size . '">

								<!-- Author Avatar -->

								<div class="' . NNRobots_List_Author::$prefix_dash . 'avatar-container ' . NNRobots_List_Author::$prefix_dash . 'avatar-size-' . $avatar_size . ' ' . NNRobots_List_Author::$prefix_dash . 'avatar-shape-' . $avatar_shape . '">';

									if ($avatar_size != 'no') {
										$code .= get_avatar($author->ID, '300');
									}

								$code .= '</div>

								<!-- Author Name -->

								<a href="' . get_author_posts_url($author->ID) . '" class="' . NNRobots_List_Author::$prefix_dash . 'author-name">' . $author->display_name . '</a>

								<!-- Author Bio -->

								<p class="' . NNRobots_List_Author::$prefix_dash . 'author-bio">' . $author->description . '</p>
							</div>';

						$code .= '</div>';

					break;

					case 'grid':

						$code .= '<div class="' . NNRobots_List_Author::$prefix_dash . 'author-grid-item">

							<div class="' . NNRobots_List_Author::$prefix_dash . 'author-info-container">

								<!-- Author Avatar -->

								<div class="' . NNRobots_List_Author::$prefix_dash . 'avatar-container ' . NNRobots_List_Author::$prefix_dash . 'avatar-size-' . $avatar_size . ' ' . NNRobots_List_Author::$prefix_dash . 'avatar-shape-' . $avatar_shape . '">';

									if ($avatar_size != 'no') {
										$code .= '<a href="' . get_author_posts_url($author->ID) . '" title="' . $author->display_name . '">' . get_avatar($author->ID, '300') . '</a>';
									}

								$code .= '</div>

								<!-- Author Name -->

								<div class="' . NNRobots_List_Author::$prefix_dash . 'author-info">
									<a href="' . get_author_posts_url($author->ID) . '" class="' . NNRobots_List_Author::$prefix_dash . 'author-name">' . $author->display_name . '</a>
								</div>

							</div>
						</div>';

					break;

					case 'carousel':

						$code .= '<li>' . get_avatar($author->ID, '300') . '

							<!-- Author Name -->

							<div class="' . NNRobots_List_Author::$prefix_dash . 'author-info">
								<a href="' . get_author_posts_url($author->ID) . '" class="' . NNRobots_List_Author::$prefix_dash . 'author-name">' . $author->display_name . '</a>
							</div>

							<!-- Latest Post -->

							<a class="' . NNRobots_List_Author::$prefix_dash . 'author-latest-post" href="' . get_permalink($posts[0]->ID) .  '">' . $posts[0]->post_title . '</a>

						</li>';

					break;

					default: // list

						$code .= '<div class="' . NNRobots_List_Author::$prefix_dash . 'author-list-item">

							<div class="' . NNRobots_List_Author::$prefix_dash . 'author-info-container" data-avatar-size="' . $avatar_size . '">

								<!-- Author Avatar -->

								<div class="' . NNRobots_List_Author::$prefix_dash . 'avatar-container ' . NNRobots_List_Author::$prefix_dash . 'avatar-size-' . $avatar_size . ' ' . NNRobots_List_Author::$prefix_dash . 'avatar-shape-' . $avatar_shape . '">';

									if ($avatar_size != 'no') {
										$code .= get_avatar($author->ID, '300');
									}

								$code .= '</div>

								<!-- Author Name -->

								<a href="' . get_author_posts_url($author->ID) . '" class="' . NNRobots_List_Author::$prefix_dash . 'author-name">' . $author->display_name . '</a>
							</div>';

						$code .= '</div>';

					break;
				}
			}

			if ( $layout == 'carousel' ) {
				$code .= '</ul></div>';
			}

		$code .= '</div>';

		if ( !$shortcode ) {
			$code .= $args['after_widget'];;
		}

		// Include slider files

		if ($layout == 'carousel') {

			wp_enqueue_script(NNRobots_List_Author::$prefix . 'slider_js', plugins_url('core/includes/flex-slider/jquery.flexslider-min.js', __FILE__), array('jquery'));
			wp_enqueue_style(NNRobots_List_Author::$prefix . 'slider_css', plugins_url('core/includes/flex-slider/flexslider.css', __FILE__));

			if ( $shortcode ) {
				$code .= '<script type="text/javascript">
				jQuery(document).ready(function($) {

					$("#' . $this_id . ' .' . NNRobots_List_Author::$prefix_dash . 'slider").flexslider({
						animationLoop: false,
						itemWidth: 210,
						itemMargin: 5,
						animation: "slide",
						directionNav: true,
					});
				});
				</script>';
			} else {

				$code .= '<script type="text/javascript">
				jQuery(document).ready(function($) {

					$("#' . $this_id . ' .' . NNRobots_List_Author::$prefix_dash . 'slider").flexslider({
						animation: "slide",
						directionNav: true,
					});
				});
				</script>';
			}
		}

		wp_enqueue_style(NNRobots_List_Author::$prefix . 'css', NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/core/css/list-authors.css');

		return $code;
	}

	/**
	 * Back-end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {

		isset($instance[ 'title' ]) ? $title = $instance[ 'title' ] : $title = __('Author List', NNRobots_List_Author::$text_domain);
		isset($instance[ 'layout' ]) ? $layout = $instance[ 'layout' ] : $layout = '';
		isset($instance[ 'num_authors' ]) ? $num_authors = $instance[ 'num_authors' ] : $num_authors = '';
		isset($instance[ 'user_roles' ]) ? $user_roles = $instance[ 'user_roles' ] : $user_roles = 'Administrator,Editor,Author';
		isset($instance[ 'exclude_users' ]) ? $exclude_users = $instance[ 'exclude_users' ] : $exclude_users = '';
		isset($instance[ 'orderby' ]) ? $orderby = $instance[ 'orderby' ] : $orderby = '';
		isset($instance[ 'order' ]) ? $order = $instance[ 'order' ] : $order = '';
		isset($instance[ 'avatar_size' ]) ? $avatar_size = $instance[ 'avatar_size' ] : $avatar_size = '';
		isset($instance[ 'avatar_shape' ]) ? $avatar_shape = $instance[ 'avatar_shape' ] : $avatar_shape = '';

		// User Roles

		global $wp_roles;
		$roles = array();

		foreach ( $wp_roles->roles as $role ) {

			if ( isset($role['capabilities']['edit_posts']) && $role['capabilities']['edit_posts'] ) {
				$roles[] = $role['name'];
			}

		}

		wp_enqueue_style(NNRobots_List_Author::$prefix . 'selectize_css',	NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/admin/css/selectize.css');
		wp_enqueue_script(NNRobots_List_Author::$prefix . 'selectize_js',	NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/admin/js/selectize.min.js');
		wp_enqueue_script(NNRobots_List_Author::$prefix . 'widget_js',		NNROBOTS_LIST_AUTHORS_PLUGIN_URL . '/admin/js/widget-settings.js');

		wp_localize_script(NNRobots_List_Author::$prefix . 'widget_js', 'nnr_al_widget_data', array(
			'prefix_dash'			=> NNRobots_List_Author::$prefix_dash,
			'post_types'			=> get_post_types(array('public' => true), 'names'),
			'user_roles'			=> $roles,
			'users'					=> get_users(array('who'=>'authors', 'number' => 500)),
		));

		include('admin/views/widget-settings.php');
	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {

		$instance = array();
		$instance['title']        = ( ! empty( $new_instance['title'] ) ) ? sanitize_text_field( $new_instance['title'] ) : '';
		$instance['layout']       = ( ! empty( $new_instance['layout'] ) ) ? sanitize_text_field( $new_instance['layout'] ) : '';
		$instance['user_roles']   = ( ! empty( $new_instance['user_roles'] ) ) ? sanitize_text_field( $new_instance['user_roles'] ) : '';
		$instance['exclude_users']= ( ! empty( $new_instance['exclude_users'] ) ) ? sanitize_text_field( $new_instance['exclude_users'] ) : '';
		$instance['orderby']      = ( ! empty( $new_instance['orderby'] ) ) ? sanitize_text_field( $new_instance['orderby'] ) : '';
		$instance['order']        = ( ! empty( $new_instance['order'] ) ) ? sanitize_text_field( $new_instance['order'] ) : '';
		$instance['num_authors']  = ( ! empty( $new_instance['num_authors'] ) ) ? sanitize_text_field( $new_instance['num_authors'] ) : '';
		$instance['avatar_size']  = ( ! empty( $new_instance['avatar_size'] ) ) ? sanitize_text_field( $new_instance['avatar_size'] ) : '';
		$instance['avatar_shape'] = ( ! empty( $new_instance['avatar_shape'] ) ) ? sanitize_text_field( $new_instance['avatar_shape'] ) : '';

		return $instance;
	}

}
